<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 *
 * @package    format_iena
 * @copyright  2018 Softia/Université lorraine
 * @author     vrignaud camille / Thomas Fradet
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
define('NO_OUTPUT_BUFFERING', true);
require_once('../../../config.php');
require_once('view/view_param_indicateur.php');
require_once('entity/course_format_iena_section_ressources.php');
require_once('entity/course_format_iena_sections.php');
require_once('entity/course_format_iena_groups.php');

global $COURSE, $DB, $USER;

// Defines the id of the course with a get parameter.
$courseid = required_param('courseid', PARAM_INT);

// Define the url of the view.
$url = new moodle_url('/course/format/iena/suivi_unit.php', array('courseid' => $courseid));

$PAGE->set_url($url);

// Getting DB information (*)  of the course.
$course = $DB->get_record('course', array('id' => $courseid), '*', MUST_EXIST);

require_login($course);
$PAGE->set_title($course->fullname);
$PAGE->set_heading($course->fullname . " – Suivi des étudiants");

$PAGE->set_pagelayout('incourse');

// Cache le menu dans le header.
$coursenode = $PAGE->navbar->add('suiviiena', null, navigation_node::TYPE_CONTAINER, null, 'suiviena');
$coursenode->make_active();
$coursenode->force_open();

require_once($CFG->libdir . '/completionlib.php');
$completion = new completion_info($course);
$context = context_course::instance($COURSE->id);

if (!has_capability('course/iena:suivi', $context = context_course::instance($courseid), $USER->id)) {
    $link = $CFG->wwwroot . '/course/view.php?id=' . $courseid;
    header("Location: {$link}");
    exit;
}

echo $OUTPUT->header();


$filters = [
    "0" => array(
        "value" => "all",
        "name" => "Tous",
    ),
    "1" => array(
        "value" => "=100%",
        "name" => "100%",
    ),
    "2" => array(
        "value" => "=0",
        "name" => "0",
    ),
    "3" => array(
        "value" => "<100",
        "name" => "<100%",
    ),
    "4" => array(
        "value" => "<50",
        "name" => "<50%",
    ),
    "5" => array(
        "value" => "<25",
        "name" => "<25%",
    ),
    "6" => array(
        "value" => ">50",
        "name" => ">=50%",
    ),
    "7" => array(
        "value" => ">25",
        "name" => ">=25%",
    ),
    "8" => array(
        "value" => "0",
        "name" => ">0%",
    ),
];

// Groupe du GET provenant du sélecteur de la page du cours, ou premier groupe de l'utilisateur, ou groupe 0 (tous les groupes).
if (isset($_GET['groupid'])) {
    $activegroupid = $_GET['groupid'];
} else {
    $activegroupid = null;
}

// TODO à supprimer si pas utilisé dans le JS (mettre dans la clause IF pour éviter un appel inutile si le GET existe).
$currentusergroupsids = array();
// Si pas de groupe dans le get ou groupe 0 (tous) mais pas le droit => premier groupe existant du user ou groupe 0 (tous).
if ($activegroupid == null || ($activegroupid == 0 && !has_capability('course/iena:suivi_edit', $context, $USER->id)) ) {
    if (count($currentusergroupsids) == 0) {
        $activegroupid = 0;
    } else {
        $currentusergroupsids = groups_get_user_groups($COURSE->id, $USER->id)[0];
        $activegroupid = $currentusergroupsids[0];
    }
}

// Groupes du cours avec id, nom et liste des id utilisateur de tous les
// membres (3 clefs d'un tableau de groupes : id, name, member).
if ($activegroupid == 0) {
    // On récupère les ids des membres.
    $groups = groups_get_all_groups($COURSE->id, 0, 0, 'g.*', true);
} else {
    // On ne récupère pas les ids des membres.
    $groups = groups_get_all_groups($COURSE->id, 0, 0, 'g.*', false);
}

$currentusergroups = [];
$activegroupname = "";
foreach ($groups as $group) {
    // Récupération du nom du group actif au passage.
    if ($group->id == $activegroupid) {
        $activegroupname = $group->name;
        $group->selected = 'selected';
    }
    foreach ($currentusergroupsids as $ugi) {
        if ($group->id == $ugi) {
            $currentusergroups[] = $group;
        }
    }
}


// Liste de tous les utilisateurs avec :
// - informations personnelles (tous les utilisateurs du cours)
// - liste des activités avec état d'achèvement (si achevées, sinon vide).
$progress = $completion->get_progress_all(
    '',
    array(),
    $activegroupid,
    'u.lastname ASC, u.firstname ASC',
    '',
    '',
    $context
);

// Formate l'achèvement d'activités pour le tableau de suivi.
// Pour chaque étudiant, on aura dans le bon ordre la liste des modules et le completionstate.
function format_progress($progress, $modules) {
    global $CFG, $COURSE;
    $students = array();
    foreach ($progress as $proginfo) {
        $progressstudent = new StdClass();
        $progressstudent->name = $proginfo->firstname." ".$proginfo->lastname;
        $progressstudent->id = $proginfo->id;
        $progressstudent->email = $proginfo->email;
        $progressstudent->progress = array();
        $progressstudent->report_link = $CFG->wwwroot . "/report/outline/user.php?id=" .
            $progressstudent->id . "&course=" . $COURSE->id . "&mode=outline";
        $progressstudent->message_link = $CFG->wwwroot . "/message/index.php?id=" . $progressstudent->id;
        foreach ($modules as $key => $module) {
            $moduleprogress = new StdClass();
            if (!isset($proginfo->progress[$module->id])) {
                $moduleprogress->completionstate = '0';
            } else {
                $moduleprogress->completionstate = $proginfo->progress[$module->id]->completionstate;
            }
            $moduleprogress->namemodule = $module->name;
            $progressstudent->progress[$key] = $moduleprogress;
        }
        $students[] = $progressstudent;
    }
    return $students;
}

// Ajoute les groupes de l'utilisateur pour l'afficher dans l'export
// Si le user a le droit d'afficher pour tous les groupes ou qu'il n'y en a pas,
// on récupère le groupe de l'utilisateur pour qu'il puisse être téléchargé dans le tableau blobal intergroupe.
if ($activegroupid == 0) {
    foreach ($progress as $prog) {
        $prog->groups = "";
        foreach ($groups as $group) {
            if (in_array($prog->id, $group->members)) {
                $prog->groups .= $group->name . " ";
            }
        }
    }
}

// Liste de toutes les activités du cours (sauf en attente de suppression). La liste est épurée et constitue un tableau d'objet.
$activities = $completion->get_activities();
$modules = [];
foreach ($activities as $activity) {
    $module = new StdClass();
    $module->id = $activity->id;
    $module->name = $activity->name;
    $displayname = format_string($activity->name, true, array('context' => $activity->context));
    $module->displayname = strlen($displayname) > 20 ? mb_substr($displayname, 0, 19, 'UTF-8').'…' : $displayname;
    $module->section = $activity->section;
    $modules[] = $module;
}

// Liste des sections du cours.
$modinfo = get_fast_modinfo($COURSE->id);
$sectionsinfoall = $modinfo->get_section_info_all();
$sections = [];

// Section du get acquise depuis le clic sur l'indicateur dans la page du cours,
// section depuis laquelle on a cliqué sur le bouton pour voir le suivi.
if (isset($_GET['sectionid'])) {
    $activesectionid = $_GET['sectionid'];
} else {
    $activesectionid = 0;
}

foreach ($sectionsinfoall as $key => $sectioninfo) {
    $section = new StdClass();
    $section->id = $sectioninfo->id;
    $section->name = $sectioninfo->name === null || $sectioninfo->name === '' ? 'Section ' . $key : $sectioninfo->name;
    if ($section->id == $activesectionid) {
        $section->selected = "selected";
    }

    $sections[] = $section;
}

// Filtre actif si il existe (si la page est rechargée pour un filtre de groupe p.ex.).
if (isset($_GET['filter'])) {
    $filter = $_GET['filter'];
} else {
    $filter = 'all';
}

// On a récupéré toutes les données, on met ça dans un tableau pour le template.
$data = array();
$data['filters'] = $filters;
$data["sections"] = $sections;
$data["groups"] = array_values($groups);
$data['current_user_groups'] = $currentusergroups;

if ($activegroupname == "") {
    $data['default_group'] = 'selected';
}

$data["modules"] = $modules;
$data["students"] = format_progress($progress, $modules);

if (has_capability('course/iena:suivi_edit', $context = context_course::instance($COURSE->id), $USER->id)) {
    $data['link_bulkcompletion'] = $CFG->wwwroot . "/course/bulkcompletion.php?id=" . $COURSE->id;
}

$data['link_classicview'] = $CFG->wwwroot . "/report/progress/index.php?course=" . $COURSE->id;

$data['data'] = array();
$data['data']["sections"] = $sections;
$data['data']["groups"] = array_values($groups);
$data['data']['current_user_groups'] = $currentusergroups;
$data['data']["modules"] = $modules;
$data['data']["students"] = format_progress($progress, $modules);
$data['data'] = json_encode($data['data']);

// If a post is sent trought the page.
if ($_POST && !isset($_POST["action"])) {
    require_once("$CFG->libdir/formslib.php");
    require_once('view/view_send_message.php');
    $usersid = $_POST["api_url"];
    $view = new view_send_message();
    echo $view->get_content($usersid);
} else if (isset($_POST['action']) && !empty($_POST['action'])) {
    return $data;
} else {
    $renderer = $PAGE->get_renderer('format_iena');
    $renderer->display_completion($data);
}

echo $OUTPUT->footer();
